/**
 * Copyright (C) 2011 Agrosense <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.isobus.type;

import java.util.List;
import nl.cloudfarming.client.isobus.model.*;
import nl.cloudfarming.client.isobus.model.Farm;
import nl.cloudfarming.client.isobus.model.IsobusPartField;
import nl.cloudfarming.client.isobus.model.ValuePresentation;
import org.openide.nodes.ChildFactory;
import org.openide.nodes.Children;
import org.openide.nodes.Node;
import org.openide.util.NbBundle;
import org.openide.util.NbBundle.Messages;

/**
 * Creates Aggregate nodes for Isobus taskdata.xml nodes
 *
 * Can handle IsoBus types only! -----------------------------
 *
 * ValuePresentation Farm CropType Worker ProductGroup Product CulturalPractice
 * Device CommentAllocation Connection DeviceAllocation DataLog
 * ProductAllocation Time TreatmentZone
 *
 *
 * ! IsobusPartField has its own Childfactory !
 *
 * Trying to create an aggregate node for any other type will result in an
 * UnsupportedOperationException!
 *
 * @author Gerben Feenstra
 */
@Messages({
    "isobus.project.taskdata.node.name.valuepresentation={0}",
    "isobus.project.taskdata.node.name.farm=Farm ({0})",
    "isobus.project.taskdata.node.name.commentallocation=CommentAllocation ({0})",
    "isobus.project.taskdata.node.name.connection=Connection",
    "isobus.project.taskdata.node.name.croptype={0} - {1}",
    "isobus.project.taskdata.node.name.culturalpractice={0} - {1}",
    "isobus.project.taskdata.node.name.datalog=DataLog",
    "isobus.project.taskdata.node.name.device={0} - {1}",
    "isobus.project.taskdata.node.name.deviceallocation=DeviceAllocation",
    "isobus.project.taskdata.node.name.product={0} - {1}",
    "isobus.project.taskdata.node.name.productallocation=ProductAllocation",
    "isobus.project.taskdata.node.name.productgroup={0} - {1}",
    "isobus.project.taskdata.node.name.time=Time ({0} - {1})",
    "isobus.project.taskdata.node.name.treatmentzone=TreatmentZone ({0})",
    "isobus.project.taskdata.node.name.worker={0} - {1}"
})
public class AggregateNodeChildFactory<I> extends ChildFactory<Node> {

    // Taskdata
    private static final String VALUEPRESENTATION_CHILDNODE_ICON = "";
    private static final String CROPTYPE_CHILDNODE_ICON = "nl/cloudfarming/client/isobus/type/isobus_taskdata_croptype_aggregate.png";
    private static final String WORKER_CHILDNODE_ICON = "nl/cloudfarming/client/isobus/type/isobus_taskdata_worker.png";
    private static final String PRODUCT_CHILDNODE_ICON = "nl/cloudfarming/client/isobus/type/isobus_taskdata_product.png";
    private static final String PRODUCTGROUP_CHILDNODE_ICON = "nl/cloudfarming/client/isobus/type/isobus_taskdata_productgroup.png";
    private static final String CULTURALPRACTICE_CHILDNODE_ICON = "";
    private static final String DEVICE_CHILDNODE_ICON = "nl/cloudfarming/client/isobus/type/isobus_taskdata_device.png";
    private static final String FARM_CHILDNODE_ICON = "nl/cloudfarming/client/isobus/type/isobus_taskdata_farm.png";
    // Task
    private static final String TASK_CHILDNODE_ICON = "";
    private static final String COMMENTALLOCATION_CHILDNODE_ICON = "";
    private static final String CONNECTION_CHILDNODE_ICON = "";
    private static final String DEVICEALLOCATION_CHILDNODE_ICON = "nl/cloudfarming/client/isobus/type/isobus_taskdata_device-allocation.png";
    private static final String DATALOG_CHILDNODE_ICON = "nl/cloudfarming/client/isobus/type/isobus_taskdata_datalog.png";
    private static final String PRODUCTALLOCATION_CHILDNODE_ICON = "";
    private static final String TIME_CHILDNODE_ICON = "nl/cloudfarming/client/isobus/type/isobus_taskdata_time.png";
    private static final String TREATMENTZONE_CHILDNODE_ICON = "nl/cloudfarming/client/isobus/type/isobus_taskdata_treatmentzone.png";
    private List<I> isobusObjects;

    public AggregateNodeChildFactory(List<I> isobusObjects) {
        this.isobusObjects = isobusObjects;
    }

    @Override
    protected boolean createKeys(List<Node> toPopulate) {
        if (isobusObjects != null && isobusObjects.size() > 0) {

            // We are sure that all objects in the list are of the same type, 
            // Checking the first object to get access to the right for-loop is much less costly
            // then doing an instanceof for each object in the list. (List of Time objects can contain 150 objects for instance).
            I checkingInstance = isobusObjects.get(0);

            // TaskData Children
            if (checkingInstance instanceof ValuePresentation) {
                for (I isobusObject : isobusObjects) {
                    ValuePresentation valuePresentation = (ValuePresentation) isobusObject;
                    String displayName = NbBundle.getMessage(AggregateNodeChildFactory.class, "isobus.project.taskdata.node.name.valuepresentation", valuePresentation.getValuePresentationId());
                    toPopulate.add(new IsobusCommonNode(Children.LEAF, valuePresentation, VALUEPRESENTATION_CHILDNODE_ICON, displayName));
                }
            } else if (checkingInstance instanceof Farm) {
                for (I isobusObject : isobusObjects) {
                    Farm farm = (Farm) isobusObject;
                    String displayName = NbBundle.getMessage(AggregateNodeChildFactory.class, "isobus.project.taskdata.node.name.farm", farm.getName());
                    toPopulate.add(new IsobusCommonNode(Children.LEAF, farm, FARM_CHILDNODE_ICON, displayName));
                }
            } else if (checkingInstance instanceof CropType) {
                for (I isobusObject : isobusObjects) {
                    CropType cropType = (CropType) isobusObject;
                    String displayName = NbBundle.getMessage(AggregateNodeChildFactory.class, "isobus.project.taskdata.node.name.croptype", cropType.getId(), cropType.getName());
                    toPopulate.add(new IsobusCommonNode(Children.LEAF, cropType, CROPTYPE_CHILDNODE_ICON, displayName));
                }
            } else if (checkingInstance instanceof Worker) {
                for (I isobusObject : isobusObjects) {
                    Worker worker = (Worker) isobusObject;
                    String displayName = NbBundle.getMessage(AggregateNodeChildFactory.class, "isobus.project.taskdata.node.name.worker", worker.getWorkerId(), worker.getWorkerDesignator());
                    toPopulate.add(new IsobusCommonNode(Children.LEAF, worker, WORKER_CHILDNODE_ICON, displayName));
                }
            } else if (checkingInstance instanceof ProductGroup) {
                for (I isobusObject : isobusObjects) {
                    ProductGroup productGroup = (ProductGroup) isobusObject;
                    String displayName = NbBundle.getMessage(AggregateNodeChildFactory.class, "isobus.project.taskdata.node.name.productgroup", productGroup.getProductGroupID(), productGroup.getProductGRoupDesignator());
                    toPopulate.add(new IsobusCommonNode(Children.LEAF, productGroup, PRODUCTGROUP_CHILDNODE_ICON, displayName));
                }
            } else if (checkingInstance instanceof Product) {
                for (I isobusObject : isobusObjects) {
                    Product product = (Product) isobusObject;
                    String displayName = NbBundle.getMessage(AggregateNodeChildFactory.class, "isobus.project.taskdata.node.name.product", product.getProductId(), product.getProductDesigNator());
                    toPopulate.add(new IsobusCommonNode(Children.LEAF, product, PRODUCT_CHILDNODE_ICON, displayName));
                }
            } else if (checkingInstance instanceof CulturalPractice) {
                for (I isobusObject : isobusObjects) {
                    CulturalPractice culturalPractice = (CulturalPractice) isobusObject;
                    String displayName = NbBundle.getMessage(AggregateNodeChildFactory.class, "isobus.project.taskdata.node.name.culturalpractice", culturalPractice.getId(), culturalPractice.getDesignator());
                    toPopulate.add(new IsobusCommonNode(Children.LEAF, culturalPractice, CULTURALPRACTICE_CHILDNODE_ICON, displayName));
                }
            } else if (checkingInstance instanceof Device) {
                for (I isobusObject : isobusObjects) {
                    Device device = (Device) isobusObject;
                    String displayName = NbBundle.getMessage(AggregateNodeChildFactory.class, "isobus.project.taskdata.node.name.device", device.getId(), device.getDesignator());
                    toPopulate.add(new IsobusCommonNode(Children.LEAF, device, DEVICE_CHILDNODE_ICON, displayName));
                }
            } // Task Children
            else if (checkingInstance instanceof CommentAllocation) {
                for (I isobusObject : isobusObjects) {
                    CommentAllocation commentAllocation = (CommentAllocation) isobusObject;
                    String displayName = NbBundle.getMessage(AggregateNodeChildFactory.class, "isobus.project.taskdata.node.name.commentallocation", commentAllocation.getId());
                    toPopulate.add(new IsobusCommonNode(Children.LEAF, commentAllocation, COMMENTALLOCATION_CHILDNODE_ICON, displayName));
                }
            } else if (checkingInstance instanceof Connection) {
                for (I isobusObject : isobusObjects) {
                    Connection connection = (Connection) isobusObject;
                    String displayName = NbBundle.getMessage(AggregateNodeChildFactory.class, "isobus.project.taskdata.node.name.connection");
                    toPopulate.add(new IsobusCommonNode(Children.LEAF, connection, CONNECTION_CHILDNODE_ICON, displayName));
                }
            } else if (checkingInstance instanceof DeviceAllocation) {
                for (I isobusObject : isobusObjects) {
                    DeviceAllocation deviceAllocation = (DeviceAllocation) isobusObject;
                    String displayName = NbBundle.getMessage(AggregateNodeChildFactory.class, "isobus.project.taskdata.node.name.deviceallocation");
                    toPopulate.add(new IsobusCommonNode(Children.LEAF, deviceAllocation, DEVICEALLOCATION_CHILDNODE_ICON, displayName));
                }
            } else if (checkingInstance instanceof DataLog) {
                for (I isobusObject : isobusObjects) {
                    DataLog dataLog = (DataLog) isobusObject;
                    String displayName = NbBundle.getMessage(AggregateNodeChildFactory.class, "isobus.project.taskdata.node.name.datalog");
                    toPopulate.add(new IsobusCommonNode(Children.LEAF, dataLog, DATALOG_CHILDNODE_ICON, displayName));
                }
            } else if (checkingInstance instanceof ProductAllocation) {
                for (I isobusObject : isobusObjects) {
                    ProductAllocation productAllocation = (ProductAllocation) isobusObject;
                    String displayName = NbBundle.getMessage(AggregateNodeChildFactory.class, "isobus.project.taskdata.node.name.productallocation");
                    toPopulate.add(new IsobusCommonNode(Children.LEAF, productAllocation, PRODUCTALLOCATION_CHILDNODE_ICON, displayName));
                }
            } else if (checkingInstance instanceof IsobusTime) {
                for (I isobusObject : isobusObjects) {
                    IsobusTime time = (IsobusTime) isobusObject;
                    String displayName = NbBundle.getMessage(AggregateNodeChildFactory.class, "isobus.project.taskdata.node.name.time", time.getTimeStart(), time.getTimeStop());
                    toPopulate.add(new IsobusCommonNode(Children.LEAF, time, TIME_CHILDNODE_ICON, displayName));
                }
            } else if (checkingInstance instanceof TreatmentZone) {
                for (I isobusObject : isobusObjects) {
                    TreatmentZone treatmentZone = (TreatmentZone) isobusObject;
                    String displayName = NbBundle.getMessage(AggregateNodeChildFactory.class, "isobus.project.taskdata.node.name.treatmentzone", treatmentZone.getTreatmentZoneDesignator());
                    toPopulate.add(new IsobusCommonNode(Children.LEAF, treatmentZone, TREATMENTZONE_CHILDNODE_ICON, displayName));
                }
            } else if (checkingInstance instanceof IsobusTask) {
                for (I isobusObject : isobusObjects) {
                    IsobusTask isobusTask = (IsobusTask) isobusObject;
                    String displayName = NbBundle.getMessage(AggregateNodeChildFactory.class, "isobus.project.taskdata.node.name.task", isobusTask.getTaskId(), isobusTask.getTaskDesignator());
                    toPopulate.add(new IsobusCommonNode(Children.LEAF, isobusTask, TASK_CHILDNODE_ICON, displayName));
                }
            } else {
                throw new UnsupportedOperationException("Trying to create an node for an unsupported type: " + checkingInstance.getClass());
            }
        }
        return true;
    }

    @Override
    protected Node createNodeForKey(Node key) {
        return key;
    }
}
